﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nollie.UnitTests;
using System.Threading.Tasks;
using Nollie.CacheTypes.Memory;
using Nollie.CacheTypes;

namespace Nollie.IntegrationTests
{
	[TestClass]
	public class MemCacheTests
	{
		private static IMemCache _cache;

		[ClassInitialize]
		public static void Initialize(TestContext context)
		{
			_cache = CacheFactory.GetMemCache("Memory");

			_cache.ClearAllPartitions();
		}
		private IPartition CreatePartition()
		{
			return _cache.GetMemPartition(Guid.NewGuid().ToString(), 10000);
		}

		[TestMethod]
		public void Clear_should_delete_all_items_within_partition()
		{
			var partition = CreatePartition();

			for (int i = 0; i < 10; i++)
			{
				var item = TestDataHelper.GetValidCacheItem(i.ToString());

				partition.Add(item);
			}

			Assert.AreEqual(10, partition.Count());

			partition.Clear();

			Assert.AreEqual(0, partition.Count());
		}

		[TestMethod]
		public void Add_item_should_be_added_to_database()
		{
			var partition = CreatePartition();

			var item = TestDataHelper.GetValidCacheItem("1");
			partition.Add(item);

			Assert.AreEqual((item.Value as Customer).Name, partition.Get<Customer>("1").Name);
		}
		[TestMethod]
		public void Get_item_from_cache_item_is_returned_if_not_expired()
		{
			var partition = CreatePartition();

			var item = TestDataHelper.GetValidCacheItem("1");
			partition.Add(item);

			Assert.AreEqual((item.Value as Customer).Name, partition.Get<Customer>("1").Name);
		}
		[TestMethod]
		public void Get_null_is_returned_if_key_is_not_found()
		{
			var partition = CreatePartition();

			Assert.AreEqual(null, partition.Get<Customer>("2"));
		}
		[TestMethod]
		public void Delete_item_from_cache()
		{
			var partition = CreatePartition();

			var item = TestDataHelper.GetValidCacheItem("1");
			partition.Add(item);

			Assert.AreEqual(1, partition.Count());

			partition.RemoveByCacheKey("1");

			Assert.AreEqual(0, partition.Count());
		}
		[TestMethod]
		public void Delete_item_from_cache_by_dependency_key()
		{
			var partition = CreatePartition();

			var item = TestDataHelper.GetValidCacheItem("1");
			item.Tags.Add("123");
			partition.Add(item);

			Assert.AreEqual(1, partition.Count());

			partition.RemoveByTag("123");

			Assert.AreEqual(0, partition.Count());
		}
		[TestMethod]
		public void Add_items_in_parallel()
		{
			var partition = CreatePartition();

			Parallel.For(0, 9999, i =>
			{
				var item = TestDataHelper.GetValidCacheItem(i.ToString());
				partition.Add(item);

				var expected = (Customer) item.Value;
				var actual = partition.Get<Customer>(i.ToString());

				Assert.AreEqual(expected.Name, actual.Name);
			}
			);
		}
		[TestMethod]
		public void Add_items_to_different_partitions_in_parallel()
		{
			var cache = CacheFactory.GetMemCache(Guid.NewGuid().ToString());

			Parallel.For(0, 10, i =>
			{
				var partition = cache.GetMemPartition(Guid.NewGuid().ToString(), 1000);

				Parallel.For(0, 100, ii =>
				{
					var item = TestDataHelper.GetValidCacheItem(ii.ToString());
					partition.Add(item);

					var expected = (Customer) item.Value;
					var actual = partition.Get<Customer>(ii.ToString());

					Assert.AreEqual(expected.Name, actual.Name);
				});
			}
			);
		}
		[TestMethod]
		public void Cache_ClearAllPartitions_should_delete_all_items_in_all_partitions()
		{
			var cache = CacheFactory.GetMemCache(Guid.NewGuid().ToString());

			var item = TestDataHelper.GetValidCacheItem("1");

			var partition1 = CreatePartition();
			partition1.Add(item);
			Assert.AreEqual(1, partition1.Count());

			var partition2 = CreatePartition();
			partition1.Add(item);
			Assert.AreEqual(1, partition1.Count());

			_cache.ClearAllPartitions();

			Assert.AreEqual(0, partition1.Count());
			Assert.AreEqual(0, partition2.Count());
		}
		[TestMethod]
		public void Cache_RemoveByTag_should_remove_item_with_tag_from_all_partitions()
		{
			var item = TestDataHelper.GetValidCacheItem("1");
			item.Tags.Add("tag:1");

			var partition1 = CreatePartition();
			partition1.Add(item);

			Assert.AreEqual(1, partition1.Count());

			var partition2 = CreatePartition();
			partition1.Add(item);

			Assert.AreEqual(1, partition1.Count());

			_cache.RemoveByTag("tag:1");

			Assert.AreEqual(0, partition1.Count());
			Assert.AreEqual(0, partition2.Count());
		}
		[TestMethod]
		public void GetMemPartition_should_return_existing_partition_if_key_already_exists()
		{
			var partitionName = Guid.NewGuid().ToString();

			var partition1 = _cache.GetMemPartition(partitionName, 100);

			var partition2 = _cache.GetMemPartition(partitionName, 100);

			Assert.AreEqual(partition1, partition2);
		}
		[ClassCleanup]
		public static void Finished()
		{
			_cache.ClearAllPartitions();
		}
	}
}
